unit NTypes;

interface

type
  TLongintArray = array of Longint;
  TStringArray = array of string;
  TSeekWhence = (swSet, swCur, swEnd);
  TStream = class(TObject)
    ReadP, EndP, Buf: PChar;
    destructor Destoroy; virtual;
    function Read(var Buf; NBytes: Cardinal): Cardinal; virtual; abstract;
    function Write(const Buf; NBytes: Cardinal): Cardinal; virtual; abstract;
    function Seek(Offset: Integer; Whence: TSeekWhence=swSet): Cardinal;
        virtual; abstract;
    function Seekable: Boolean; virtual; abstract;
    function Size: Cardinal; virtual; abstract;
    function ReadLine(var S: string): Boolean;
    function ReadLines: TStringArray;
  end;
  TFileMode = set of (fmRead, fmWrite, fmReadWrite, fmTruncate);
  TDirMode = set of (dmDirectory, dmReadByOwner, dmWriteByOwner);
  TDir = record
    Name: string;
    Size: Int64;
    ATime: Int64;
    MTime: Int64;
    Mode: TDirMode;
  end;
  TTime = record
    Year: Integer;
    Mon: 1..12;
    MDay: 1..31;
    Hour: 0..23;
    Min: 0..59;
    Sec: 0..61;
    WDay: 0..6;
  end;
  TMChar = array[0..7] of Char;

implementation

destructor TStream.Destoroy;
begin
  if Buf <> nil then
    FreeMem(Buf);
end;

function TStream.ReadLine(var S: string): Boolean;
const
  bufsize = 8192;
var
  len, dlen: Integer;
  sp: PChar;
  foundlf: Boolean;
begin
  if Buf = nil then
    GetMem(Buf, bufsize);

  dlen := 0;
  foundlf := False;
  while not foundlf do
  begin
    if ReadP = EndP then
    begin
      len := Read(Buf^, bufsize);
      if len = 0 then
      begin
        Result := dlen <> 0;
        Exit;
      end;
      ReadP := Buf;
      EndP := Buf + len;
    end;

    sp := ReadP;
    while (ReadP < EndP) and (not foundlf) do
    begin
      foundlf := ReadP^ = #10;
      Inc(ReadP);
    end;

    SetLength(S, dlen + (ReadP - sp));
    Move(sp^, S[dlen+1], ReadP - sp);
    Inc(dlen, ReadP - sp);
  end;
  Result := True;
end;

function TStream.ReadLines: TStringArray;
var
  i: Integer;
  s: string;
begin
  SetLength(Result, 1024);
  i := 0;
  while ReadLine(s) do
  begin
    if i = Length(Result) then
      SetLength(Result, i + i div 4);
    Result[i] := s;
    Inc(i);
  end;
  SetLength(Result, i);
end;

end.
